# File Maker for Photo Color Adjustment 16/8bit Version
# Ver. 4.00
# add psedo flat-field correction algorism
# Silent & Verbose Version
# (c)2020-22 Ohnishi, Yasuo
from ij import IJ, ImagePlus 
from ij import plugin
from ij import WindowManager as WM
from ij.io import DirectoryChooser
from ij.io import OpenDialog
from ij.gui import GenericDialog
from ij.plugin import ChannelSplitter
from ij.plugin import Duplicator
from ij.process import ImageConverter
from loci.plugins import BF
import os
import re
import sys
import io
import datetime
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')
def convValue(Value, Mode):
#   Converting Luminance Value each in 8, 16, 32bit mode
    if Mode == "16-bit":
        Value = Value * 256
    elif Mode == "32-bit":
        Value = Value / 255
    else:
        Value = Value
    return Value
#----------------
def psedo_Flat_field_C(imp_g0, imp_b0, imp_rg0, sigma, Mode, folder, \
        filename, parm_subtractBv, parm_leveladj, parm_correct_fac):
### Options for developping
    if parm_subtractBv == False: # No Subtraction B when B is much higher than G
        test = "A"  # select test "A" or "B"
    else:  # Subtraction B when B is much higher than G
        test = "B"  # select test "A" or "B"
    writingout = False  # select writingout True or False
    if parm_leveladj != "No":
        meanadjustment = True # select mean adjustment True or False
    else:
        meanadjustment = False # select mean adjustment True or False
    ####
    imp_width = imp_g0.getWidth()
    imp_height  = imp_g0.getHeight()
    #IJ.showMessage(str(imp_width))
    swidth = int(imp_width / 4)
    sheight = int(imp_height / 4)
    imp_g1 = Duplicator().run(imp_g0) # for Gaussian image of G
    imp_b1 = Duplicator().run(imp_b0) # for Gaussian image of B
    imp_b2 = Duplicator().run(imp_b0) # for corrected B image
    imp_rg1 = Duplicator().run(imp_rg0) # for Gaussian image of (R+G)/2
    ImageConverter(imp_g1).convertToGray16()
    ImageConverter(imp_b1).convertToGray16()
    ImageConverter(imp_b2).convertToGray16()
    ImageConverter(imp_rg1).convertToGray16()
    stat_g = imp_g1.getStatistics()
    stat_b = imp_b1.getStatistics()
    mean_g = stat_g.mean
    mean_b = stat_b.mean
    mean_dif = mean_g - mean_b  # if g > b: value+ if g < b: value-  
    """
    if meanadjustment == True: # adjust level difference of G and B
        IJ.run(imp_b1, "Add...", "value=" + str(mean_dif))
    """
    imp_g1.resize(swidth, sheight, 'bilinear')
    imp_b1.resize(swidth, sheight, 'bilinear')
    imp_rg1.resize(swidth, sheight, 'bilinear') # for periphery correction
    IJ.run(imp_g1, "Gaussian Blur...", "sigma=" + str(sigma))
    IJ.run(imp_b1, "Gaussian Blur...", "sigma=" + str(sigma))
    IJ.run(imp_rg1, "Gaussian Blur...", "sigma=" + str(sigma))
    imp_g1.resize(imp_width, imp_height, 'bilinear')
    imp_b1.resize(imp_width, imp_height, 'bilinear')
    imp_rg1.resize(imp_width, imp_height, 'bilinear')
    """
        imp_g1: G ch. Gaussan blur image
        imp_b1: B ch. Gaussan blur image
        imp_rg1: (R+ G)/2 Gaussan blur image (for periphery correction)
    """
    if meanadjustment == True: # adjust level difference of G and B Ver. 3.53
        stat_g1 = imp_g1.getStatistics()
        stat_b1 = imp_b1.getStatistics()
        if parm_leveladj == "Auto":
            if stat_g1.mean < stat_b1.mean: # adjust level difference of G and B Gaussian image (Ver. 3.52)
                IJ.run(imp_g1, "Add...", "value-" + str(stat_g1.mean - stat_b1.mean))
        elif parm_leveladj == "Manual": # Manual adjustment of correction amount of B
            IJ.run(imp_g1, "Add...", "value-" + str(convValue(parm_correct_fac, Mode)))
                # add correction value "parm_correct_fac" 
    ic = plugin.ImageCalculator()
    imp_dif_gb = ic.run("Subtract create", imp_g1, imp_b1) 
                # imp_dif_gb: darker region of Gaussian B than G
    imp_c_b0 = ic.run("Add create", imp_b2, imp_dif_gb) 
                # image correction of B (subtract yellowed value from B)
    imp_dif_b_rg = ic.run("Subtract create", imp_b1, imp_rg1) 
                # B - (R+ G)/2 for making periphery correction image
    if test =="B":
        # subtract higher region of B than G
        imp_dif_bg = ic.run("Subtract create", imp_b1, imp_g1) # No window made
        imp_c_b = ic.run("Subtract create", imp_c_b0, imp_dif_bg) # No window made
    else:
        imp_c_b = imp_c_b0 # for test "A"
    stat_c_b = imp_c_b.getStatistics()
    """
    if stat_c_b.mean > stat_g.mean:
        IJ.run(imp_c_b, "Subtract...", "value=" + str(int(stat_c_b.mean - stat_g.mean)))
         not good!
    """
    imp_c_b_prph = ic.run("Subtract create", imp_c_b, imp_dif_b_rg) 
                 # make periphery correction image
    if writingout == True:
        IJ.saveAsTiff(imp_dif_gb, folder + filename +"_imp_Gaussian-dif_gb.tif")
        if test =="B":
            IJ.saveAsTiff(imp_dif_bg, folder + filename +"_imp_Gaussian-dif_bg.tif")
    if Mode == "8-bit":
        ImageConverter(imp_c_b).convertToGray8()
        ImageConverter(imp_c_b_prph).convertToGray8()
    elif Mode == "16-bit":
        ImageConverter(imp_c_b).convertToGray16()
        ImageConverter(imp_c_b_prph).convertToGray16()
    return imp_c_b, imp_c_b_prph, imp_b1, imp_g1, imp_rg1
#---------------------------------
def make_Gaussian_fg_mask(impG, imp_c_b, imp9, folder, filename):
    """
    This routine is making mask for foreground layer in psedo flat-field correction algorism
    """
    debugmode = False
    ic = plugin.ImageCalculator()
    imp8_cB = Duplicator().run(imp_c_b) # corrected B ch.
    imp8_G = Duplicator().run(impG) # G ch.
    stat8_cB = imp8_cB.getStatistics()
    statG = impG.getStatistics()
    #IJ.showMessage("imp8_cB Mean: " + str(stat8_cB.mean / 256) + " G Mean: " + str(statG.mean / 256))
    if statG.mean < stat8_cB.mean: #adjust B value relative to G
        IJ.run(imp8_G, "Add...", "value=" + str(int(stat8_cB.mean - statG.mean)))
    imp8_gG = ic.run("Subtract create", imp8_G, imp8_cB)
    imp8_G.close()
        # extract more green region 
    stat = imp8_gG.getStatistics()
    if debugmode == True:
        IJ.saveAsTiff(imp8_cB, folder + filename +"_R+128_imp_c_b.tif")
        IJ.saveAsTiff(imp8_gG, folder + filename +"_R+128_imp8_gG.tif")
        IJ.showMessage("imp8_gG Max: " + str(stat.max / 256) + " Mean: " + str(stat.mean / 256))
    IJ.run(imp8_gG, "Multiply...", "value=10")
    IJ.run(imp8_gG, "Invert", "") # more green region  > more black (opaque)
    if debugmode == True:
        IJ.saveAsTiff(imp8_gG, folder + filename +"_R+128_imp8_gG_inverted.tif")
    # G value higher > more transparent / G value lower > less transparent
    imp8_G = Duplicator().run(impG) # G ch.
    if statG.mean > stat8_cB.mean: #adjust B value relative to G
        IJ.run(imp8_G, "Subtract...", "value=" + str(int(statG.mean - stat8_cB.mean)))
    imp8_1 = ic.run("Subtract create", imp8_cB, imp8_G) 
                  #extract more blue region
    # B value much higher region > more transparent
    stat8_1 = imp8_1.getStatistics()
    if debugmode == True:
        IJ.showMessage("imp8_1 Max: " + str(stat1.max / 256) + " Mean: " + str(stat1.mean / 256))
    IJ.run(imp8_1, "Multiply...", "value=10") 
    """
    stat8_1 = imp8_1.getStatistics()
    stat8_gG = imp8_gG.getStatistics()
    if stat8_1.mean < stat8_gG.mean:
        IJ.run(imp8_1, "Add...", "value=" + str(stat8_gG - stat8_1))
    """
    if debugmode == True: 
        IJ.saveAsTiff(imp8_1, folder + filename +"_R+128_imp8_1.tif")
    #IJ.run(imp8_gG, "Multiply...", "value=0.5") 
    #IJ.run(imp8_1, "Multiply...", "value=0.5") 
    imp8_2 = ic.run("Max create", imp8_gG, imp8_1) 
    # IJ.run(imp9, "Invert...","")
    imp8 = ic.run("Max create", imp8_2, imp9) 
    IJ.run(imp8, "Gamma...", "value=0.5")
    IJ.run(imp8, "Gaussian Blur...", "sigma=20")
    imp8_1.close()
    imp8_2.close()
    return imp8
    """
    imp8:
        region more blue > more transparent
        region more green > more opaque
    """
#####################################################
def RGBmanipulation(folder,filename,parm):
#  input Parameters
    if parm is not None:
 # if parm is not null object, then...
        Mode, parm_Binary, parm_Gain, parm_Shadow, Show, parm_Prph, parm_FGGain, parm_FFCorrection, parm_FFC_sigma, parm_Gmixrate, parm_subtractBv, parm_leveladj, parm_correct_fac = parm
        print(Mode, parm_Binary, parm_Gain, parm_Shadow, Show, parm_Prph, parm_FGGain) 
#     Mode: Image bit depth 16-bit or 8-bit
#     Show: Show working windows or not
#     parm_Binary: Threshold of fore / background
#     parm_Gain: Lightness of Distance adjusting Layer
#     parm_Shadow: Threshold of Dark part adjustment image
#     parm_Prph: Threshold of Periphery adjustment image
#     parm_FGGain: Threshold of Foreground mask

# Always Open file with Bio-format Plugin
    imps = BF.openImagePlus(folder + filename) # Open file with Bio-format Plugin
    imp0 = imps[0] #imp0: Original Image 
    if Mode == "Auto":
        bitdepth = imp0.getBitDepth()
        if bitdepth == 8:
            Mode = "8-bit"
        elif bitdepth ==16:
            Mode = "16-bit"
        elif bitdepth ==32:
            Mode = "32-bit"
        else:
            Mode = "16-bit"
    if Show == "Yes":
 # Windows Showing Mode
        imp0.show()
#------
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in impsCh: # it needs to convert Grayscale image
        if Mode == "8-bit":
            ImageConverter(i).convertToGray8()
        elif Mode == "16-bit":
            ImageConverter(i).convertToGray16()
        else:
            ImageConverter(i).convertToGray32()
## R ch.
    impR =  impsCh[0]
    IJ.saveAsTiff(impR, folder + filename +"_R.tif")
## G ch.
    impG =  impsCh[1]
    IJ.saveAsTiff(impG, folder + filename +"_G.tif")
## B ch. & B x 0.3
    impB =  impsCh[2]
    IJ.saveAsTiff(impB, folder + filename +"_B.tif")
# Show Windows
    imp0.close()
    if Show == "Yes":
        impR.show()
        impG.show()
        impB.show()
#--- preparetion for making layer images / to make (R+G)/2 image
#         & periphery correction layer
# G ch. x 0.5
#        Make impG05  as G x 0.5 Image
    impG05 = Duplicator().run(impG)
    if Show == "Yes":
        impG05.show()
    IJ.run(impG05, "Multiply...", "value=0.5")
# R ch. x 0.5
#        Make impR05  as R x 0.5 Image
    impR05 = Duplicator().run(impR)
    if Show == "Yes":
        impR05.show()
    IJ.run(impR05, "Multiply...", "value=0.5")
# B x 0.5 + R x 0.5
    ic = plugin.ImageCalculator()
    imp5 = ic.run("Add create", impR05, impG05) # No window made
    #-----save periphery correction layer (no psedo FF correction)
    IJ.saveAsTiff(imp5, folder + filename +"_G+R.tif")
    if Show == "Yes": #Hide the image windows to close
        imp5.show()
        impR05.hide()
        impG05.hide()
    impR05.close()
    impG05.close()

#--- making foreground layer image
#  executing psedo Flat-Field correction
    if parm_FFCorrection == True:
        imp_c_b, imp_c_b_prph, imp_Gauss_b1, imp_Gauss_g1, imp_Gauss_rg1 = psedo_Flat_field_C(impG, impB, imp5, parm_FFC_sigma, Mode, folder, filename, parm_subtractBv, parm_leveladj, parm_correct_fac) 
        #-----save periphery correction layer (apply psedo FF correction)
        IJ.saveAsTiff(imp_c_b_prph, folder + filename +"_G+R.tif")

##      Make impB03 as B x 0.3 Image > B:G ratio variable
    if parm_FFCorrection == True: # new algorithm
        impB03 = Duplicator().run(imp_c_b)
        Bmixrate = 1.0 - parm_Gmixrate
        strValueB =  "value=" + str(Bmixrate)
        strValueG =  "value=" + str(parm_Gmixrate)
    else: # existing algorithm
        impB03 = Duplicator().run(impB)
        strValueB =  "value=0.3"
        strValueG =  "value=0.7"
    statB03 = impB03.getStatistics()
    IJ.run(impB03, "Multiply...", strValueB)
    if Show == "Yes":
        impB03.show()
# G ch. x 0.7
##        Make impG07 as G x 0.7 Image
    impG07 = Duplicator().run(impG)
    statG07 = impG07.getStatistics()
    if statG07.mean < statB03.mean:
        IJ.run(impG07, "Add...", "value=" + str(statB03.mean - statG07.mean))
        # adjust G value for mixing to B
    IJ.run(impG07, "Multiply...", strValueG)
    if Show == "Yes":
        impG07.show()
# B x 0.3 + G x 0.7
    ic = plugin.ImageCalculator()
    imp3 = ic.run("Add create", impB03, impG07) # No window made
    IJ.saveAsTiff(imp3, folder + filename +"_B_foreground.tif")
        # even in new alogorithm G mixing is needed to improve texture
    imp3.close()
#--- making background layer image
# (G ch.+30) x 0.7
##        Make impG30x07 as (G+30) x 0.7 Image
    if parm_FFCorrection == False:  # no applying Flat-field
        impG30x07 = Duplicator().run(impG)
        if Show == "Yes":
            impG30x07.show()
        # Gain for background layer image
        # appropriate gamma is /1.45(= 0.69) for perceptive TRC image
        if parm_FGGain == 128: # in case of perceptive gamma TRC image.
            IJ.run(impG30x07, "Gamma...", "value=" + str(0.69))
        else: # in case of linear image.
            IJ.run(impG30x07, "Gamma...", "value=" + str(0.7))
        IJ.run(impG30x07, "Multiply...", "value=0.7")
    ## B x 0.3 + (G+30) x 0.7 + x(parm_Gain)
        ic = plugin.ImageCalculator()
        imp4 = ic.run("Add create", impB03, impG30x07) # No window made
    #  add B x 0.3 + (G+30) x 0.7 + x
        IJ.run(imp4, "Add...", "value=" + str(convValue(parm_Gain, Mode))) # add Value parm_Gain
        if Show == "Yes":
  #Hide the image windows to close
            impG30x07.hide()
        impG30x07.close()
    else:  # applying Flat-field
        imp4 = Duplicator().run(imp_c_b)
    IJ.saveAsTiff(imp4, folder + filename +"_B_Background.tif")
    imp4.close()
    if Show == "Yes":
        impG07.hide()
        impB03.hide()
    impG07.close()
    impB03.close()
#--- making Periphery Adjustment Mask
# Periphery Adjustment Mask: B - ((R+G)/2-10)
    impGR = Duplicator().run(imp5) #imp5: (G+R)/2 image
    if Show == "Yes":
        impGR.show()
#    if parm_FGGain != 128:
#        IJ.run(impB, "Gamma...", "value=" + str(1/2.2))
#        IJ.run(impGR, "Gamma...", "value=" + str(1/2.2))
    IJ.run(impGR, "Subtract...", "value="+ str(convValue(parm_Prph, Mode))) # subtract Value parm_Prph /10
    ic = plugin.ImageCalculator()
    if parm_FFCorrection == False:
        imp2GR = ic.run("Subtract create", impB, impGR) #impB - impGR:(G+R/2)
    else:
        imp2GR = ic.run("Subtract create", imp_c_b, impGR) #impB - impGR:(G+R/2)
    if Show == "Yes":
        imp2GR.show()
    IJ.run(imp2GR, "Multiply...", "value=4") # Save the Subtracted image x 4
    IJ.saveAsTiff(imp2GR, folder + filename +"_Periphery_Adjst_Mask.tif") 
    if Show == "Yes":
 #Hide the image windows to close
        imp2GR.hide()
        impGR.hide()
    imp2GR.close()
 #Close the images
    impGR.close()
#
#--- making dark parts correction layer 
# Dark Parts 0 - parm_Shadow
    imp6 = Duplicator().run(imp5) #imp5: (G+R)/2 image, imp6: dark part image
    if Show == "Yes":
        imp6.hide()
    IJ.run(imp6, "Add...", "value=" + str(convValue(255 - parm_Shadow, Mode))) 
    IJ.run(imp6, "Subtract...", "value=" + str(convValue(255 - parm_Shadow, Mode)))
    IJ.saveAsTiff(imp6, folder + filename +"_Dark.tif")
    if Show == "Yes":
 #Hide the image windows to close
        imp5.hide()
        imp6.hide()
    imp5.close()
 #Close the images
    imp6.close()


#--- making background correction mask 
# Make Binary Mask: Threshold R parm_Binary
    imp9 = Duplicator().run(impR)
    if Show == "Yes":
        imp9.show()
    imp9.getProcessor().threshold(int(convValue(parm_Binary, Mode))) # Make binary Mask
    if Mode != "8-bit":
        IJ.run(imp9, "Multiply...", "value=" + str(convValue(1, Mode))) # 16, 32bit image > Multiply 256 ^ 1 or 2
        IJ.run(imp9, "Add...", "value=" + str(convValue(1, Mode) - 1)) # if 16-bit > make Max Value 65535
    imp9.updateAndDraw()
    IJ.saveAsTiff(imp9, folder + filename +"_Binary_Mask.tif")



#--- making foreground correction mask 
# R + parm_FGGain(128) ## mask image for foreground correction layer
    if parm_FFCorrection == False:  # No applying Flat-field
        imp8 = Duplicator().run(impR)
        IJ.run(imp8, "Add...", "value=" + str(convValue(parm_FGGain, Mode)))
    else:   # applying Flat-field
        imp8 = make_Gaussian_fg_mask(impG, imp_c_b, imp9, folder, filename)
    if Show == "Yes":
        imp8.show()
    IJ.saveAsTiff(imp8, folder + filename +"_R+128.tif")
    if Show == "Yes":
        imp8.hide()
    imp8.close()



    if Show == "Yes":
 #Hide the imp9 window to close
        imp9.hide()
        impB.hide()
    imp9.close()
    impR.close()
    impG.close()
    impB.close()

#------------------------------------------------
# Get parameters from the input DialogBox 
def getParameters(path):
    oldparms = read_in(path)
    """
    print "oldparms"
    print oldparms["parm_Binary"]
    print oldparms["parm_FGGain"]
    """
    gd = GenericDialog("Enter Parameters")
    TRCType = ["Perceptual", "Linear"]
    """
    # IJ.showMessage["parm_FGGain" + str(oldparms["parm_FGGain"])]
    """
    if int(oldparms["parm_FGGain"]) == 128:
        gd.addChoice("Image TRC Type", TRCType, TRCType[0])
    else:
        gd.addChoice("Image TRC Type", TRCType, TRCType[1])
    
    gd.addNumericField("Background Mask Threshold (0 to 255)",float(oldparms["parm_Binary"]), 0)
    gd.addNumericField("Background Image Gain (Max to 100)", float(oldparms["parm_Gain"]), 0)
    gd.addNumericField("Shadow Area Threshold (0 to 255)", float(oldparms["parm_Shadow"]), 0)
    gd.addNumericField("Periphery Adjst Threshold (-10 to 10)", float(oldparms["parm_Prph"]), 0)
    gd.addMessage("-----------------------------------")
    gd.addMessage("[Extended Psedo Flat-Field correction parameters]")
    gd.addCheckbox("apply psedo Flat-Field correction", eval(oldparms["parm_FFCorrection"]) )
    gd.addNumericField("Sigma for Flat-Field correction", float(oldparms["parm_FFC_sigma"]), 1)
    gd.addNumericField("G mix rate to B (with FF Correction)", float(oldparms["parm_Gmixrate"]), 1)
    leveladj = ["No","Auto","Manual"]
    if oldparms["parm_leveladj"] == "Auto":
        gd.addChoice("Correction Amount adjustment", leveladj, leveladj[1])
    elif oldparms["parm_leveladj"] == "Manual":
        gd.addChoice("Correction Amount adjustment", leveladj, leveladj[2])
    else:
        gd.addChoice("Correction Amount adjustment", leveladj, leveladj[0])
    gd.addNumericField("Incremental Amount (in Manual 0-255)", float(oldparms["parm_correct_fac"]), 0)
    gd.addCheckbox("Subtract Pixel Value when B value is higher than G", eval(oldparms["parm_subtractBv"]))
    gd.addMessage("----------")
    showWin = ["Yes","No"]
    gd.addChoice("Show working windows?", showWin, showWin[0])
    gd.addMessage("Recommended Parameters for Linear Images\n 60 for Background Mask Threshold\n 3 for Shadow Area Threshold\n 0 for Periphery Adjst Threshold")
    gd.showDialog()
    parm_Mode   = "Auto"
    TRCselected   = gd.getNextChoice()
    if TRCselected == "Perceptual":
        parm_FGGain = 128
    else:
        parm_FGGain = 56
    parm_Binary   = gd.getNextNumber()
    parm_Gain = gd.getNextNumber()
    parm_Shadow = gd.getNextNumber()
    parm_Prph = gd.getNextNumber()
    parm_FFCorrection = gd.getNextBoolean() #get value of checkbox
    parm_FFC_sigma = gd.getNextNumber()
    parm_Gmixrate = gd.getNextNumber()
    parm_leveladj  = gd.getNextChoice()
    parm_correct_fac = gd.getNextNumber()
    parm_subtractBv = gd.getNextBoolean() #get value of checkbox
    parm_Show   = gd.getNextChoice()

    """
         parm_leveladj, parm_correct_fac
    """
    if parm_Binary > 255: #Set Max Value
        parm_Binary = 255
    if parm_Gain > 100:
        parm_Binary = 100
    if parm_Shadow > 255: #Set Max Value
        parm_Shadow = 255
        if parm_Gmixrate > 1.0:
            parm_Gmixrate = 1.0
        elif parm_Gmixrate < 0.0:
            parm_Gmixrate = 0.0
    if gd.wasCanceled():  
        parm_Mode   = "Stop"
        IJ.log("Canceled")
        return str(parm_Mode), parm_Binary, parm_Gain, parm_Shadow,str(parm_Show), \
            parm_Prph, parm_FGGain, parm_FFCorrection, parm_FFC_sigma, parm_Gmixrate, \
            parm_subtractBv, parm_leveladj, parm_correct_fac
    write_out(path, parm_Binary, parm_Gain, parm_Shadow, parm_Prph, \
            parm_FGGain, parm_FFCorrection, parm_FFC_sigma, parm_Gmixrate, \
            parm_subtractBv, parm_leveladj, parm_correct_fac)
    return str(parm_Mode), parm_Binary, parm_Gain, parm_Shadow, str(parm_Show), \
        parm_Prph, parm_FGGain, parm_FFCorrection, parm_FFC_sigma, parm_Gmixrate, \
        parm_subtractBv, parm_leveladj, parm_correct_fac
#----------------
def write_out(path, parm_Binary, parm_Gain, parm_Shadow, parm_Prph, \
            parm_FGGain, parm_FFCorrection, parm_FFC_sigma, parm_Gmixrate, \
            parm_subtractBv, parm_leveladj, parm_correct_fac):
    wo_parm = dict(parm_Binary=parm_Binary, parm_Gain=parm_Gain, parm_Shadow=parm_Shadow, \
        parm_Prph=parm_Prph, parm_FGGain=parm_FGGain, parm_FFCorrection=parm_FFCorrection, \
        parm_FFC_sigma=parm_FFC_sigma, parm_Gmixrate=parm_Gmixrate, \
        parm_subtractBv=parm_subtractBv, parm_leveladj=parm_leveladj, \
        parm_correct_fac=parm_correct_fac)
#  -- writing "filename_parameter.txt" file
    writing_file0 = open(path  + '_parameter.txt','a')
    # new_file_number = int(file_number) + 1
    # i = 0
    for key in wo_parm.keys():
        writing_file0.write(key + ": " + str(wo_parm[key]) +"\n")
    timenow = datetime.datetime.now()
    writing_file0.write("date time: " + timenow.strftime('%Y/%m/%d/%H:%M:%S') +"\n")
    writing_file0.close()
#-------------
def read_in(path):
    existing_parms = {}
    existing_parms["existing"] = "No"
        # define dic type V.
    is_file = os.path.isfile(path + '_parameter.txt')
    if is_file:
        # reading existing parameters 
        with open(path + '_parameter.txt','r') as read_file:
            lines = read_file.readlines()
            # IJ.showMessage(str(lines[0]))
            if len(lines) >= 12:
                for l in range(len(lines)-12, len(lines)-1):
                    eachline = lines[l].strip()
                    i = eachline.find(":")
                    key = eachline[:i]
                    value = eachline[i+2:]
                    existing_parms[key] = value
                    existing_parms["existing"] = "Yes"
    # print "existing: ", existing_parms["existing"], "Lines: ", len(lines)
    if existing_parms["existing"] == "No":
        existing_parms["parm_FFC_sigma"] = 20.0
        existing_parms["parm_subtractBv"] = "False"
        existing_parms["parm_leveladj"] = "Auto"
        existing_parms["parm_Shadow"] = 50.0
        existing_parms["parm_Prph"] = 10.0
        existing_parms["parm_correct_fac"] = 0.0
        existing_parms["parm_Binary"] = 128.0
        existing_parms["parm_Gmixrate"] = 0.2
        existing_parms["parm_Gain"] = 0.0
        existing_parms["parm_FFCorrection"] = "False"
        existing_parms["parm_FGGain"] = 128
    return existing_parms        
#-------------
# Main
od = OpenDialog("Choose a file")
folder = od.getDirectory()
file = od.getFileName()
if file:
    path = folder + file
    IJ.log(path)
    IJ.log(folder)
    IJ.log(file)
    parm = getParameters(path)
    print ("Parm[0]=", parm[0])
    if parm[0] != "Stop":
        RGBmanipulation(folder,file,parm)
    IJ.log("Finish")
else:
    IJ.log("Job Canceled")
#    IJ.run("Quit")
